/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;

import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.name.BotanicalName;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.model.Resources;
import eu.etaxonomy.taxeditor.store.VocabularyStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 * @author n.hoffmann
 */
public class PreferencesUtil {
	private static final Logger logger = Logger
			.getLogger(PreferencesUtil.class);

	/**
	 * 
	 * @return
	 */
	public static IPreferenceStore getPreferenceStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}
	
	/**
	 * 
	 * @return
	 */
	private static String getNameCodePreference() {
		try{
			return getPreferenceStore().getString(Resources.CODE_PREFERENCE);
		}catch(NullPointerException e){
			logger.warn("PreferenceStore was not available. That is OK if you are running a unit test. Setting ICBN as default code.");
			return Resources.CODE_PREFERENCE_ICBN;
		}
	}
	
	/**
	 * 
	 * @param preferredCode
	 */
	public static void setNomenclaturalCode(String preferredCode) {
		getPreferenceStore().setValue(Resources.CODE_PREFERENCE, preferredCode);
	}
	
	/**
	 * 
	 */
	public static void setDefaultNomenclaturalCode() {
		setNomenclaturalCode(Resources.DEFAULT_CODE_PREFERENCE);
	}

	/**
	 * 
	 * @return
	 */
	public static String getPreferredNomenclaturalCodeAsString() {
		String nameCodePreference = getNameCodePreference();
		
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return "International Code of Botanical Nomenclature (ICBN)";
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return "International Code of Zoological Nomenclature (ICZN)";
		}
		return null;
	}
	
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		String nameCodePreference = getNameCodePreference();
				
		if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICBN)) {
			return NomenclaturalCode.ICBN;
		} else if (nameCodePreference.equals(Resources.CODE_PREFERENCE_ICZN)) {
			return NomenclaturalCode.ICZN;
		}
		return null;
	}
	
	
	/**
	 * Returns a new name object that conforms to the nomenclatural
	 * code specified in user preferences. 
	 * 
	 * @return
	 */
	public static NonViralName<?> getInstanceOfPreferredNameClass() {
		NomenclaturalCode code = getPreferredNomenclaturalCode();
		
		// Check whether name code preference needs to be initialized
		if (code == null) {
			setDefaultNomenclaturalCode();
		}
		
		if (code.equals(NomenclaturalCode.ICBN)) {
			return BotanicalName.NewInstance(null);
		} else if (code.equals(NomenclaturalCode.ICZN)) {
			return ZoologicalName.NewInstance(null);			
		}
		return NonViralName.NewInstance(null);
	}

	/**
	 * Generic method to get term preferences for a term vocabulary
	 * 
	 * @param <T>
	 * @param initialTerms
	 * @return
	 */
	public static <T extends DefinedTermBase> List<T> getPreferredTerms(Set<T> initialTerms){
		
		// set default for non existent preferences
		defaultTerms(initialTerms);
				
		List<T> preferredTerms = new ArrayList<T>();
		
		for (T term : initialTerms){
			if(getPreferenceStore().getBoolean(getPreferenceKey(term))){
				preferredTerms.add(term);
			}
		}
		
		return preferredTerms;		
	}
	
	private static <T extends DefinedTermBase> boolean defaultTerms(Set<T> initialTerms){
		// set default for non existen preferences
		for(T term : initialTerms){
			if(getPreferenceStore().getBoolean(term.getClass().getName())){
				break;
			}else{
				getPreferenceStore().setValue(term.getClass().getName(), true);
				setPreferredTerms(new ArrayList(initialTerms), initialTerms);
				break;
			}
		}
		return true;
	}
	
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<SpecimenTypeDesignationStatus> getPreferredSpecimenTypeDesignationStatus() { return getPreferredTerms(VocabularyStore.getSpecimenTypeDesignationStatus());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<TaxonRelationshipType> getPreferredTaxonRelationshipTypes() { return getPreferredTerms(VocabularyStore.getTaxonRelationshipTypes());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<Feature> getPreferredFeatures() { return getPreferredTerms(VocabularyStore.getFeatures());}

	/** @return a <code>List</code> containing user preferred <code>Rank</code> terms */
	public static List<Rank> getPreferredRanks() { return getPreferredTerms(VocabularyStore.getRanks());}
	
	/** @return a <code>List</code> containing user preferred  <code>PresenceAbsenceTermBase</code> terms */
	public static List<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPreferredPresenceAbsenceTerms(){ return getPreferredTerms(VocabularyStore.getPresenceAbsenceTerms());}
	
	/** @return a <code>List</code> containing user preferred  <code>NomenclaturalStatusType</code> terms */
	public static List<NomenclaturalStatusType> getPreferredNomenclaturalStatusTypes(){	return getPreferredTerms(VocabularyStore.getNomenclaturalStatusTypes());}
	
	/** @return a <code>List</code> containing user preferred  <code>NameRelationshipType</code> terms */
	public static List<NameRelationshipType> getPreferredNameRelationshipTypes(){ return getPreferredTerms(VocabularyStore.getNameRelationshipTypes());}
	
	
	/**
	 * Generic method to set term preferences
	 * 
	 * @param preferredTerms
	 * @param initalTerms
	 */
	public static <T extends DefinedTermBase> void setPreferredTerms(List<T> preferredTerms, Set<T> initalTerms){
		for(TermBase term : initalTerms){
			getPreferenceStore().setValue(getPreferenceKey(term), preferredTerms.contains(term));
		}
	}
	
	/**
	 * Construct a unique key using the terms uuid
	 * 
	 * @param term
	 * @return
	 */
	private static String getPreferenceKey(TermBase term) {
		return term.getClass().getName()
				. concat(".")
				. concat(term.getUuid().toString());
	}
}
